function internalWeights = generate_internal_weights(nInternalUnits, ...
                                                  connectivity)
% GENERATE_INTERNAL_WEIGHTS creates a random reservoir for an ESN
%  
% inputs:
% nInternalUnits = the number of internal units in the ESN
% connectivity \in [0,1], says how many weights should be non-zero 
%
% output:
% internalWeights = matrix of size nInternalUnits x nInternalUnits
% internalWeights(i,j) = value of weight(synapse) from unit i to unit j
% internalWeights(i,j) might be different from internalWeights(j,i)

%
% Version 1.0, April 30, 2006
% (c) Dan Popovici
% Revision 1, Feb 23, 2007, H. Jaeger

% supress any diagnostic messages when computing the largest eigenvalue
opts.disp = 0 ; 

success = 0 ;                                               
while success == 0
    % following block might fail, thus we repeat until we obtain a valid
    % internalWeights matrix
    try,
        internalWeights = sprand(nInternalUnits, nInternalUnits, connectivity);
        internalWeights(internalWeights ~= 0) = internalWeights(internalWeights ~= 0)  - 0.5;
        maxVal = max(abs(eigs(internalWeights,1,'LM',opts)));
        internalWeights = internalWeights/maxVal;
        success = 1 ;
    catch,
        success = 0 ; 
    end
end
