function [] = plotTraces(landscapeMat, lowerGrayVal, upperGrayVal, logFlag,...
    x_Traces, y_Traces, FigNr, showTraces )

% This helper function plots a gray value plot for the matrix 
% landscapeMat, with a color of black for any matrix entry less or 
% equal to lowerGrayVal, and a color of white for any matrxi entry 
% greater or equal than upperGrayVal. The grayshades in between correspond to 
% a linear (set logFlag = 0) or logarithmic (logFlag = 1) scale. In addition 
% to the grayshade landscape, this function inserts T traces corresponding to the 
% T columns in the matrices x_Traces and y_Traces which contain the 
% x and y values of the trace points (only the parts of the traces that
% fall within the unit square are displayed). FigNr is the matlab figure 
% window number into which the plots go, showTraces = 0 supresses plotting the
% traces (else set to 1).
% 
% For further explanation see the paper by Jaeger et al, Optimization and 
% Applications of Echo State Networks with Leaky Integrator Neurons. 
% Neural Networks, April 2007 

% Created by H. Jaeger February 21, 2007. For conditions of use, see main
% routine "stochOpt" in this package.


if FigNr > 0
    figure(FigNr); clf; set(FigNr, 'Position', [304   686   267   229]);
end

gridN = size(landscapeMat,1)-1;

tmp = landscapeMat;
tmp(tmp < 0 | tmp > upperGrayVal) = upperGrayVal;
tmp(tmp < lowerGrayVal) = lowerGrayVal;
if logFlag
    tmp = log10(tmp);
end
maxVal = max(max(tmp)); minVal = min(min(tmp));
tmp = (tmp - minVal) / (maxVal - minVal);
tmp = max(ceil(64 * tmp), 1);
image([0 1.0],[0 1], tmp); hold on; 
contour(0:(1/gridN):1,0:(1/gridN):1, tmp, 'k');
for R = showTraces
    line(x_Traces(1,R), y_Traces(1,R), 'Marker', 'o','MarkerSize',6,...
        'Color','black','MarkerFaceColor','black' );
    line(x_Traces(:,R), y_Traces(:,R),  'Color','black', 'LineWidth', 3);
    line(x_Traces(:,R), y_Traces(:,R),  'Color','white');
    line(x_Traces(end,R), y_Traces(end,R), 'Marker', '*','MarkerSize',8, 'Color','white');
end
set(gca,'YDir','normal');
set(gca,'XLim',[0 1 ], 'YLim', [0 1]);
colormap(bone);
text(0.1, 0.9, sprintf('black = min = \n%g', lowerGrayVal), 'FontSize', 7);
text(0.1, 0.65, sprintf('white = max = \n%g  \n(or n.d.)', upperGrayVal), 'FontSize', 7);

